ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c FORTRAN77 code
c File: LR-Chan.for
c
c Date of last revision: April, 14, 1993
c
c by K.S. Chan
c   Dept. of Statistics and Actuarial Science
c   Univ. of Iowa, Iowa City, IA52242
c   U.S.A. 	 
c
c This program computes the percentage points of the LRT for
c Threshold Autoregressions. The theory is outlined in Chapter 5 of the
c book and in the paper by K.S. Chan (1991).
c   "Percentage Points of Likelihood Ratio Tests for Threshold Autoregressions", 
c   Journal of the Royal Statistical Society, B 53(3), 691-696.
c
c  To use the program, compile it by f77 and then run it. 
c     
c  Below is an example: (The file is called percentlrt.f in this example) 
c
c % f77 percentlrt.f
c percentlrt.f:
c  MAIN:
c        func:
c        gammp:
c        gser:
c        gcf:
c        zbrent:
c        gammln:
c        anormal:
c        func1:
c        erf:
c        qromb:
c        trapzd:
c        polint:
c % a.out
c R tilde = [a*100 percentile, b*100 percentile]
c Please enter a, the a*100 percentile of  standard Normal distribution,
c  the significance level and the autoregressive order p  (in free format)
c 0.25 -0.6745 0.01 2
c percentage point is     17.60
c Warning: the following IEEE floating-point arithmetic exceptions 
c occurred in this program and were never cleared: 
c Inexact;  Underflow; 
c
c I consider the warning harmless.
c The line  0.25 -0.6745 0.01 2  is input from the keyboard.
c Here, R~=[25 percentile, 75 percentile], -0.6745 is the 25 percentile
c of the standard Normal; 0.01 is the significance level and the autoregressive
c order is 2 in this example.
c
c
c Note that subroutines typed in captial letters are taken from the book 
c entitled "Numerical Receipes", referenced in the aforementioned article.
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      common p,sig,ran,sper
      external func,func1
      parameter (x2=200.,tol=.0001)
      write(6,99)
99    format(" R tilde = [a*100 percentile, b*100 percentile]")           
      write(6,101)
101   format(' Please enter a, the a*100 percentile of ', 
     *' standard Normal distribution, '
     *,' the significance level and the autoregressive order p ',
     * ' (in free format)')
      read(5,*)sper,ap,sig,ip 
      bp=-ap
      call qromb(func1,ap,bp,ran)
      p=ip
      x1=(p+1)+(2*(p+1))**.5
      z=zbrent(func,x1,x2,tol)
      write(6,100)z
100   format(' percentage point is' ,f10.4)
      stop
      end
      real function func(x)
      common p,sig,ran,sper
      ainte=-log(sper/(1.-sper))
      d=(p+1.)
      temp=((p-1)*ainte+2*ran)
      ip=p     
      if (ip .eq. 0) temp=ainte
      func=exp(gammln(d/2))*2.**(d/2)
      func=2./func*x**(d/2.-1.)*exp(-x/2.)*(x/(p+1)-1)
      func=1-exp(-func*temp)-sig
      if (ip .eq. 0) then
      y=x**.5
      pi=3.1415926
      func=(2/pi)**.5*exp(-x/2.)*(ainte*y-ainte/y+1./y)-sig
      endif
      return
      end
      FUNCTION GAMMP(A,X)
      IF(X.LT.0..OR.A.LE.0.)PAUSE
      IF(X.LT.A+1.)THEN
        CALL GSER(GAMSER,A,X,GLN)
        GAMMP=GAMSER
      ELSE
        CALL GCF(GAMMCF,A,X,GLN)
        GAMMP=1.-GAMMCF
      ENDIF
      RETURN
      END
      SUBROUTINE GSER(GAMSER,A,X,GLN)
      PARAMETER (ITMAX=100,EPS=3.E-7)
      GLN=GAMMLN(A)
      IF(X.LE.0.)THEN
        IF(X.LT.0.)PAUSE
        GAMSER=0.
        RETURN
      ENDIF
      AP=A
      SUM=1./A
      DEL=SUM
      DO 11 N=1,ITMAX
        AP=AP+1.
        DEL=DEL*X/AP
        SUM=SUM+DEL
        IF(ABS(DEL).LT.ABS(SUM)*EPS)GO TO 1
11    CONTINUE
      PAUSE 'A too large, ITMAX too small'
1     GAMSER=SUM*EXP(-X+A*LOG(X)-GLN)
      RETURN
      END
      SUBROUTINE GCF(GAMMCF,A,X,GLN)
      PARAMETER (ITMAX=100,EPS=3.E-7)
      GLN=GAMMLN(A)
      GOLD=0.
      A0=1.
      A1=X
      B0=0.
      B1=1.
      FAC=1.
      DO 11 N=1,ITMAX
        AN=FLOAT(N)
        ANA=AN-A
        A0=(A1+A0*ANA)*FAC
        B0=(B1+B0*ANA)*FAC
        ANF=AN*FAC
        A1=X*A0+ANF*A1
        B1=X*B0+ANF*B1
        IF(A1.NE.0.)THEN
          FAC=1./A1
          G=B1*FAC
          IF(ABS((G-GOLD)/G).LT.EPS)GO TO 1
          GOLD=G
        ENDIF
11    CONTINUE
      PAUSE 'A too large, ITMAX too small'
1     GAMMCF=EXP(-X+A*ALOG(X)-GLN)*G
      RETURN
      END
      FUNCTION ZBRENT(FUNC,X1,X2,TOL)
      PARAMETER (ITMAX=100,EPS=3.E-8)
      A=X1
      B=X2
      FA=FUNC(A)
      FB=FUNC(B)
      IF(FB*FA.GT.0.) PAUSE 'Root must be bracketed for ZBRENT.'
      FC=FB
      DO 11 ITER=1,ITMAX
        IF(FB*FC.GT.0.) THEN
          C=A
          FC=FA
          D=B-A
          E=D
        ENDIF
        IF(ABS(FC).LT.ABS(FB)) THEN
          A=B
          B=C
          C=A
          FA=FB
          FB=FC
          FC=FA
        ENDIF
        TOL1=2.*EPS*ABS(B)+0.5*TOL
        XM=.5*(C-B)
        IF(ABS(XM).LE.TOL1 .OR. FB.EQ.0.)THEN
          ZBRENT=B
          RETURN
        ENDIF
        IF(ABS(E).GE.TOL1 .AND. ABS(FA).GT.ABS(FB)) THEN
          S=FB/FA
          IF(A.EQ.C) THEN
            P=2.*XM*S
            Q=1.-S
          ELSE
            Q=FA/FC
            R=FB/FC
            P=S*(2.*XM*Q*(Q-R)-(B-A)*(R-1.))
            Q=(Q-1.)*(R-1.)*(S-1.)
          ENDIF
          IF(P.GT.0.) Q=-Q
          P=ABS(P)
          IF(2.*P .LT. MIN(3.*XM*Q-ABS(TOL1*Q),ABS(E*Q))) THEN
            E=D
            D=P/Q
          ELSE
            D=XM
            E=D
          ENDIF
        ELSE
          D=XM
          E=D
        ENDIF
        A=B
        FA=FB
        IF(ABS(D) .GT. TOL1) THEN
          B=B+D
        ELSE
          B=B+SIGN(TOL1,XM)
        ENDIF
        FB=FUNC(B)
11    CONTINUE
      PAUSE 'ZBRENT exceeding maximum iterations.'
      ZBRENT=B
      RETURN
      END
      FUNCTION GAMMLN(XX)
      REAL*8 COF(6),STP,HALF,ONE,FPF,X,TMP,SER
      DATA COF,STP/76.18009173D0,-86.50532033D0,24.01409822D0,
     *    -1.231739516D0,.120858003D-2,-.536382D-5,2.50662827465D0/
      DATA HALF,ONE,FPF/0.5D0,1.0D0,5.5D0/
      X=XX-ONE
      TMP=X+FPF
      TMP=(X+HALF)*LOG(TMP)-TMP
      SER=ONE
      DO 11 J=1,6
        X=X+ONE
        SER=SER+COF(J)/X
11    CONTINUE
      GAMMLN=TMP+LOG(STP*SER)
      RETURN
      END
      function anormal(x)
      anormal=1/(2.*3.1415926)**.5*exp(-x*x/2.)
      end
      function func1(x)
      s1=(erf(x/2**.5)+1)/2
      s2=-x*exp(-x*x/2.)/(2.*3.1415926)**.5+s1
      s3=-exp(-x*x/2.)/(2*3.1415926)**.5
      sa=(s1+s2-((s1-s2)**2+4*s3**2)**.5)/2
      a=(s2-sa)/(s1+s2-2*sa)
      b=(s1-sa)/(s1+s2-2*sa)
      c=-2.*s3/(s1+s2-2*sa)
      t=anormal(x)
      func1=1/(2*sa*(1-sa))*(a+x*x*b+c*x)*t
      return
      end
      FUNCTION ERF(X)
      IF(X.LT.0.)THEN
        ERF=-GAMMP(.5,X**2)
      ELSE
        ERF=GAMMP(.5,X**2)
      ENDIF
      RETURN
      END
      SUBROUTINE QROMB(FUNC,A,B,SS)
      PARAMETER(EPS=1.E-6,JMAX=20,JMAXP=JMAX+1,K=5,KM=4)
      DIMENSION S(JMAXP),H(JMAXP)
      H(1)=1.
      DO 11 J=1,JMAX
        CALL TRAPZD(FUNC,A,B,S(J),J)
        IF (J.GE.K) THEN
          L=J-KM
          CALL POLINT(H(L),S(L),K,0.,SS,DSS)
          IF (ABS(DSS).LT.EPS*ABS(SS)) RETURN
        ENDIF
        S(J+1)=S(J)
        H(J+1)=0.25*H(J)
11    CONTINUE
      PAUSE 'Too many steps.'
      END
      SUBROUTINE TRAPZD(FUNC,A,B,S,N)
      IF (N.EQ.1) THEN
        S=0.5*(B-A)*(FUNC(A)+FUNC(B))
        IT=1
      ELSE
        TNM=IT
        DEL=(B-A)/TNM
        X=A+0.5*DEL
        SUM=0.
        DO 11 J=1,IT
          SUM=SUM+FUNC(X)
          X=X+DEL
11      CONTINUE
        S=0.5*(S+(B-A)*SUM/TNM)
        IT=2*IT
      ENDIF
      RETURN
      END
      SUBROUTINE POLINT(XA,YA,N,X,Y,DY)
      PARAMETER (NMAX=10) 
      DIMENSION XA(N),YA(N),C(NMAX),D(NMAX)
      NS=1
      DIF=ABS(X-XA(1))
      DO 11 I=1,N 
        DIFT=ABS(X-XA(I))
        IF (DIFT.LT.DIF) THEN
          NS=I
          DIF=DIFT
        ENDIF
        C(I)=YA(I)
        D(I)=YA(I)
11    CONTINUE
      Y=YA(NS)
      NS=NS-1
      DO 13 M=1,N-1
        DO 12 I=1,N-M
          HO=XA(I)-X
          HP=XA(I+M)-X
          W=C(I+1)-D(I)
          DEN=HO-HP
          IF(DEN.EQ.0.)PAUSE
          DEN=W/DEN
          D(I)=HP*DEN
          C(I)=HO*DEN
12      CONTINUE
        IF (2*NS.LT.N-M)THEN
          DY=C(NS+1)
        ELSE
          DY=D(NS)
          NS=NS-1
        ENDIF
        Y=Y+DY
13    CONTINUE
      RETURN
      END
